#! /usr/bin/env python

import glob, os, sys

files = glob.glob("temps*")
t = { 
        'alloc'   : [],
        'init'    : [],
        'copy'    : [],
        'free'    : [],
        'add'     : [],
        'verif'   : [],
        'free'    : [],
        'compile' : [],
        'total'   : []
    }

for f in files:
    v = f[6:-4]
    with open(f) as fIn:
        for l in fIn:
            ll = l.split()
            t[ll[0]].append([v, float(ll[1]), None])

for p in t:
    t[p].sort(key=lambda q: q[0])
    for q in range(len(t[p])):
        if t[p][q][1]> 0:
            t[p][q][2] = t[p][0][1]/t[p][q][1]
        else:
            t[p][q][2] = 0.0

l = '{:>16s}{:>12s}{:>10s}'.format('Version', 'Temps', 'Speedup')
print (l, '\n')
l = [ '{:>16s}{:12.5f}{:10.2f}'.format(q[0][2:], q[1], q[2]) for q in t['total'] ]
print ('\n'.join(l), '\n')


nv = []
nt = []
ns = []
for q in t['total']:
    nv.append(q[0][2:].replace('_','\n'))
    nt.append(q[1])
    ns.append(q[2])

try:
    import matplotlib.pyplot as plt
    graphics = True
except:
    graphics = False

if graphics:
    fig, (ax1, ax2) = plt.subplots(2, 1, figsize=(10,16))

    xmax = len(nv)
    tmax = max(nt)
    smax = max(ns)

    xlabel = "Version"
    xticks = range(1, xmax+1)
    xlabels = nv
    ax1.bar(xticks, nt, width=0.5)
    ax1.set_xlim(left=0.5, right=float(xmax+0.5))
    ax1.set_ylim(bottom=0.0, top=tmax*1.1)
    ax1.set_ylabel('CPU Time (s)')
    ax1.xaxis.set_ticks(xticks)
    ax1.xaxis.set_ticklabels(xlabels)
    ax1.grid()

    ax2.bar(xticks, ns, width=0.5)
    ax2.set_xlim(left=0.5, right=float(xmax+0.5))
    ax2.set_ylim(bottom=0.0, top=smax*1.1)
    ax2.set_xlabel(xlabel)
    ax2.set_ylabel('Speedup')
    ax2.xaxis.set_ticks(xticks)
    ax2.xaxis.set_ticklabels(xlabels)
    ax2.grid()

    fig.set_size_inches(6, 7)
    plt.savefig("speedups" + ".pdf")
