#include "timer.hxx"
#include <iomanip>
#include <sys/stat.h>
#include <map>
#include <vector>

static std::vector<Timer> Timers;

int AddTimer(const char *name)
{
  Timers.push_back(Timer(name));
  return Timers.size() - 1;
}

Timer & GetTimer(int n)
{
  if (n < 0)
     return Timers[Timers.size()+n];
  else
     return Timers[n];
}

#include <fstream>

void PrintTimers(std::ostream &f)
{
  double T_total = GetTimer(-1).elapsed();
  
  f << "Temps de calcul:\n";
  for (std::vector<Timer>::iterator it=Timers.begin(); it != Timers.end(); it++) {
    Timer & t = *it;
    if (t.name() == "total")
      continue;
    f << std::setw(15) << t.name() << ": " << std::setw(13);
    if (t.elapsed() > 0.0 && t.elapsed() < 1e-3)
      f << std::scientific; 
    else
      f << std::fixed;
    
    f << std::setprecision(3) << t.elapsed() << " s"
      << std::setw(13) << std::setprecision(2) << std::fixed
      << t.elapsed() * 100.0/T_total << " %" << std::endl;
  }
  f << "        _______________________________________\n";
  f << std::setw(15) << "total" << ": " << std::setw(13)
      << std::setprecision(3) << T_total << " s"
      << std::setw(13) << std::setprecision(2) << std::fixed
      << 100.0 << " %" << std::endl << std::endl;
}

void SaveTimers(const char *version)
{
  std::string fname = "temps_";
  fname += version;
  fname += ".dat";

  std::ofstream f(fname.c_str());
  for (std::vector<Timer>::iterator it=Timers.begin(); it != Timers.end(); it++) {
    Timer & t = *it;
    f << t.name() << " " << t.elapsed() << std::endl;
  }
}
