#! /usr/bin/env python

import os, sys, subprocess, platform, argparse

parser = argparse.ArgumentParser()
parser.add_argument('-t', '--type', default='Release',
                    choices=['Debug', 'Release', 'RelWithDebInfo'])
args = parser.parse_args()

e = os.environ.copy()
p = platform.system()
if p == 'Windows':
  gen = '-GNinja'
  compileCmd = ['ninja', 'install']
elif p == 'Linux':
  gen = '-GUnix Makefiles'
  compileCmd = ['make', '--no-print-directory', 'install']

base = os.getcwd()

srcDir = os.path.join(base, 'src')
buildDir = os.path.join(base, 'build')
installDir = os.path.join(base, 'install')

cmake_params = ['-DCMAKE_BUILD_TYPE=' + args.type]
cmake_params.append('-DCMAKE_INSTALL_PREFIX=' + installDir)
cmake_params.append(gen)

if not os.path.exists(buildDir):
  os.makedirs(buildDir)

configureCmd = ['cmake'] + cmake_params + [srcDir]
err = subprocess.call(configureCmd, cwd=buildDir, env=e)
if err == 0:
  err = subprocess.call(compileCmd, cwd=buildDir, env=e)



