#! /usr/bin/env python
# -*- coding: utf-8 -*-

try:
    unicode('')
except NameError:
    unicode = str

import os, sys, argparse, numpy, time
from subprocess import *

versions = ['CPU', 'CPU_OpenMP', 'Cuda', 'OpenCL']
parser = argparse.ArgumentParser()
parser.add_argument('--version', nargs='*', choices=versions, default=versions)
args = parser.parse_args()

baseDir = os.path.dirname(os.path.realpath(__file__))
installDir = os.path.join(baseDir, 'install')

user = os.environ['USER']

job = '''#!/bin/bash

#SBATCH --job-name=GPU1_@USER@
#SBATCH --output=output_@K@_@VERSION@.txt
#SBATCH --error=output_@K@_@VERSION@.txt
#SBATCH --partition=gpu
#SBATCH --gres=gpu:1
#SBATCH --account=ams301
#SBATCH --time=00:05:00
#SBATCH --ntasks=1

@OPENMP@

## load modules
module purge
module load gcc
module load cuda

echo "___________________________"
echo "version : @VERSION@"

## execution
@COMMANDE@ ./images/ecureuil.png
'''

k = 0
for v in args.version:

  commande = code = os.path.join(installDir, 'image_' + v)

  k += 1
  s = job
  s = s.replace("@VERSION@", v)
  s = s.replace("@K@", str(k))
  s = s.replace("@USER@", user)
  s = s.replace("@COMMANDE@", commande)
  if v == "CPU_OpenMP":
    s = s.replace("@OPENMP@", 
        "#SBATCH --cpus-per-task=4\n\nexport OMP_NUM_THREADS=4\n")
  else:
    s = s.replace("@OPENMP@", "")

  with open('job_' + v + '.sh', 'w') as f:
    f.write(s)

  call(['sbatch', 'job_'+ v + '.sh'])

