#!  /usr/bin/env python3

import os, sys, subprocess, argparse, platform, shutil

def manage():

  parser = argparse.ArgumentParser()
  parser.add_argument('--compiler', '-c', choices = ['gnu', 'intel'], default='gnu')
  args = parser.parse_args()

  p = platform.system()
  print(p)
  if p == 'Windows':
    gen = '-GNinja'
    compileCmd = ['ninja', 'install']
  else:
    gen = '-GUnix Makefiles'
    compileCmd = ['make', '--no-print-directory', 'install']
  e = os.environ.copy()

  baseDir = os.getcwd()

  for p in ['cuda', 'opencl']:

    if p == 'cuda':
      e['CXX'] = 'nvcc'
      e['CC'] = 'nvcc'
    else:
      e['CXX'] = 'g++'
      e['CC'] = 'gcc'
    srcDir = os.path.join(baseDir, 'src', p)

    buildDir = os.path.join(baseDir, 'build', p)
    installDir = os.path.join(baseDir, 'install')
 
    cmake_params = []
    if p == 'opencl' and 'OCL_ROOT' in e:
      OCL_ROOT = e['OCL_ROOT']
      cmake_params.append('-DOpenCL_LIBRARY=' + os.path.join(OCL_ROOT, 'lib64', 'libOpenCL.so'))
      cmake_params.append('-DOpenCL_INCLUDE_DIR=' + os.path.join(OCL_ROOT, 'include'))

    cmake_params.append('-DCMAKE_INSTALL_PREFIX=' + installDir)
    cmake_params.append(gen)

    if not os.path.exists(buildDir):
      os.makedirs(buildDir)

    configureCmd = ['cmake'] + cmake_params + [srcDir]
    print(' '.join(configureCmd))
    err = subprocess.call(configureCmd, cwd=buildDir, env=e)
    if err == 0:
      err = subprocess.call(compileCmd, cwd=buildDir, env=e)

if __name__ == '__main__':
  manage()

